from Products.CMFCore  import CMFCorePermissions
from types import StringType, DictType
import os.path

from content_driver.ContentType import lookupContentType
from debug import log
from utils import pathFor, unique

_modes = {
    'v' : { 'key': 'v',
            'signature' : 'view',
            'template' : '%(name)s_view.pt',
            'mode'     : 'r',
            'action'   : { 'id': 'view',
                           'name': 'View',
                           'permissions': (CMFCorePermissions.View,) 
                       },
            },
    'e' : { 'key': 'e',
            'signature' : 'edit',
            'template' : '%(name)s_edit_form.pt',
            'mode'     : 'w',
            'action'   : { 'id': 'edit',
                           'name': 'Edit',
                           'permissions': (CMFCorePermissions.ModifyPortalContent,) 
                           },
            },
    'm' : { 'key': 'm',
            'signature' : 'edit',
            'template' : '%(name)s_meta_form.pt',
            'mode'     : 'w',
            'action'   : { 'id': 'metadata',
                           'name': 'Properties',
                           'permissions': (CMFCorePermissions.ModifyPortalContent,) 
                           },

            },
    'V' : { 'key': 'V',
            'signature' : 'validate',
            'template' : 'validate_%(name)s_edit.py',
            'mode'     : 'w',
            },
    }
    
class Generator:
    def __init__(self, klass, field):
        self.klass = klass
        self.field = field
        self.form  = field.FormInfo()

        self.args = {
            'portal_type' : getattr(klass, 'portal_type', klass.__name__),
            'required'    : self.field.required == 1 and "required" or '',
            'label'       : self.form.label,
            'id'          : self.field.name,
            'description' : self.form.description,
            'accessor'    : self.field.accessor,
            'validateAs'  : self.form.validator,
            'attributes'  : self.form.attributes
            }


        self.name = self.__class__.__name__.lower()

    def templateFor(self, view, postfix=''):
        if postfix != '' and not postfix.startswith('_'):
            postfix = "_%s" % postfix
            
        target = "%s_%s%s" % (self.name, view, postfix)
        template = _templates.get(target, None)
        if not template:
            #log("No template for %s" % target)
            target = "%s_%s%s" % ("generator", view, postfix)
            template = _templates.get(target, '')


        return template

    def view(self, **kwargs):
        """This should be uniform across most types"""
        return self.templateFor('view') % self.args

    def edit(self, **kwargs):
        start = self.templateFor('edit', 'start') % self.args
        body  = self.templateFor('edit', 'body') % self.args
        end   = self.templateFor('edit', 'end') % self.args
        return (start, body, end)

    def validate(self, **kwargs):
        if self.form.validator is None: return ''
        result = []
        result.append("validator.addField(%(id)r, %(validateAs)r" % self.args)
        
        #Options
        if self.field.required: result.append('required=1')
        else: result.append('required=0')
        result.append(')\n')
        return ','.join(result)

    
class TextArea(Generator):
    def __init__(self, klass, field):
        Generator.__init__(self, klass, field)
        args = self.args
        args['rows'] = self.form.rows
        args['cols'] = self.form.columns        

class LinesArea(TextArea):
    def __init__(self, klass, field):
        TextArea.__init__(self, klass, field)

class Password(Generator):
    def __init__(self, klass, field):
        Generator.__init__(self, klass, field)
        args = self.args
        args['confirm'] = self.form.confirm

    def edit(self, **kwargs):
        return self.templateFor('edit') % self.args
    


class Complex(Generator):
    def __init__(self, klass, field):
        Generator.__init__(self, klass, field)
        args = self.args
        args['rows'] = self.form.rows
        args['cols'] = self.form.columns        

    def edit(self, **kwargs):
        body  = self.templateFor('edit', 'body') % self.args
        format = self.generateFormat()
        file  = self.templateFor('edit', 'file') % self.args
        footer = self.templateFor('edit', 'footer') % self.args
        return (body, format, file, footer)

    def generateFormat(self):
        ''' Render the format template for each allowed content-type
        '''
        types = []
        for type_name in self.field.allowable_content_types:
            ct = lookupContentType(type_name)
            types.append(ct)

        
        start = self.templateFor('format', 'start') % self.args

        types = unique(types)
        body = []
        for type in types:
            self.args['mime-type']  = type.getMimeType()
            self.args['mime-label'] = type.name
            self.args['mime-id']    = type.getMimeType().replace('/', '_')
            body.append(self.templateFor('format', 'body') % self.args) 
            del self.args['mime-type']
            del self.args['mime-label']
            del self.args['mime-id']
            
        end = self.templateFor('format', 'end') % self.args

        body = ''.join(body)
        return ''.join((start, body, end))
    
class Date(Generator):
    def validate(self, **kwargs):
        return ""
        
class Keywords(Generator):

    def makeJavascript(self):
        return self.templateFor('widget', 'javascript') % self.args

    def makeKeywords(self):
        return self.templateFor('widget', 'body') % self.args
    
    def edit(self, **kwargs):
        self.args['index'] = self.form.index
        js = self.makeJavascript()
        body = self.makeKeywords()
        del self.args['index']
        return (js, body)

class Language(Generator):
    pass

class Boolean(Generator):
    pass

class Id(Generator):
    def edit(self, **kwargs):
        return self.templateFor('edit', 'body') % self.args

class Slot(Generator):
    def __init__(self, klass, field):
        Generator.__init__(self, klass, field)
        self.args['slot_metal'] = self.form.slot_metal

class File(Generator):
    def edit(self, **kwargs):
        return self.templateFor('edit', 'body') % self.args

class SingleSelection(Generator):
    def __init__(self, klass, field):
        Generator.__init__(self, klass, field)

        
    def edit(self, **kwargs):
        args = self.args

        # this covers both radio and pulldown formats
        start = self.templateFor(self.form.format, 'start') % args
        body = []

        body = self.templateFor(self.form.format, 'body') % args
        end = self.templateFor(self.form.format, 'end') % args            

        body = ''.join(body)
        return ((start, body, end))

class MultiSelection(SingleSelection):
    def __init__(self, klass, field):
        SingleSelection.__init__(self, klass, field)
        self.args['size'] = self.form.size


fieldTypes = {
    'stringinfo'      : Generator,
    'textareainfo'    : TextArea,
    'linesinfo'       : LinesArea,
    'richforminfo'    : Complex,
    'datetimeinfo'    : Date,
    'keywordsinfo'    : Keywords,
    'languageinfo'    : Language,
    'booleaninfo'     : Boolean,
    'singleselectioninfo' : SingleSelection,
    'slotinfo'        : Slot,
    'idinfo'          : Id,
    'fileinfo'        : File,
    'passwordinfo'    : Password,
    'multiselectioninfo' : MultiSelection,
    }


class FormGenerator:
    def getTargetName(self, klass, template_dir, name, mode):
        args = mode.copy()
        args['name'] = name
        
        targetName  = args['template'] % args
        targetName = targetName.lower()
        return os.path.join(template_dir, targetName)
    
    def getTarget(self, klass, template_dir, name, mode):
        targetName = self.getTargetName(klass, template_dir, name, mode)
        fp = open(targetName, "w")
        return fp

    def templateFor(self, view, postfix=''):
        if postfix != '' and not postfix.startswith('_'):
            postfix = "_%s" % postfix
            
        target = "%s%s" % (view, postfix)
        return _templates.get(target, '')

    def Header(self, fp, klass, generators, args, mode):
        text = self.templateFor(mode['signature'], "header")
        fp.write(text % args)

    def Footer(self, fp, klass, generators, args, mode):
        text = self.templateFor(mode['signature'], "footer")
        fp.write(text % args)

    def generateField(self, fp, klass, generator, args, mode):
        method = getattr(generator, mode['signature'], None)
        if not method:
            log("No method %s for %s" %(mode['signature'], generator.field.name))
            return

        result = method()
        if type(result) == type(''):
            result = [result]
        for r in result:
            fp.write(str(r))


    def computeModes(self, modes):
        result = []
        if modes == None:
            modes = _modes.values()
            
        #If we are passed a string, look up the mode in _modes
        #else append the dict itself as a new mode
        for mode in modes:
            if type(mode) == StringType:
                d = _modes[mode]
            elif type(mode) == DictType:
                d = mode
            else:
                raise TypeError("""must pass the name of a valid mode,
                or a mode dict""")
            
            result.append(d)
        return result
    
    def generateViews(self, klass, targetDir, modes=None):
        #Get a list of mode dicts
        generatorModes = self.computeModes(modes)
        baseName = klass.__name__

        actions = []
        
        for mode in generatorModes:
            def hasModes(field, mode=mode):
                return mode['key'] in field.generateMode and \
                       mode['mode'] in field.mode
            
            fields = klass.type.filterFields(hasModes)

            generators = []
            for field in fields:
                form = field.FormInfo()
                if form.visible == -1: continue

                generators.append(fieldGeneratorFactory(klass, field))

            args = {
                'description' : klass.__doc__,
                'type'        : baseName,
                'portal_type' : getattr(klass, 'portal_type', baseName),
                }

            #establish a target for this mode
            fp = self.getTarget(klass, targetDir, baseName, mode)

            #Generate to this target
            self.Header(fp, klass, generators, args, mode)
            for generator in generators:
                self.generateField(fp, klass, generator, args, mode)
            self.Footer(fp, klass, generators, args, mode)
            fp.close()


            #Optionally create a action annotation for this mode
            if mode.has_key('action'):
                action = mode['action'].copy()
                action['action'] = os.path.splitext(os.path.basename(fp.name))[0]
                actions.append(action)

        return tuple(actions)
    
def fieldGeneratorFactory(klass, field):
    form = field.FormInfo()
    factory = fieldTypes.get(form.__class__.__name__.lower(), Generator)
    return factory(klass, field)

def loadTemplate(name):
    target = pathFor('templates', name)
    text = open(target, 'r').read()
    return text

def loadTemplates():
    dir = pathFor('templates')
    files = os.listdir(dir)
    for template in files:
        #Sloppy
        if template.startswith('.') or template.startswith('#') or template.startswith("CVS"): continue
        if template.endswith('~'): continue
        _templates[template] = loadTemplate(template)


_templates = {}
loadTemplates()

_fg = FormGenerator()
generateViews = _fg.generateViews
